/* ********************************************************
    dfmax.cpp

    Author = Hideyuki Harada
    Nihon Univ. Tani Research Group


    Basic Algorithm for resolving MaxClique problem
    designed for "g++ 3.3.3" with "LEDA 4.5"
   ******************************************************** */


#include<LEDA/ugraph.h>
#include<LEDA/node_set.h>
#include<time.h>
#include<iostream.h>

using namespace leda;

ugraph G;          // undirected graph

node_set Qmax(G);    // node set for MaxClique

int Branch_count;  // number of counted SST branch


void dfmax(node_set R, node_set Q);


//*************************************************


int main(void){

  int i;
  float T = used_time();
  node_set R(G), Q(G);
  node v;

  G.read("graph.gw");

  // initialize
  Q.clear();
  forall_nodes(v, G){
    R.insert(v);
  }

  Qmax.clear();

  // start procedure
  dfmax(R, Q);


  // show result
  
  cout << endl;
  cout << "number of nodes = " << G.number_of_nodes() << endl;
  cout << "number of edges = " << G.number_of_edges() << endl;
  cout << "used time = " << used_time(T) << endl;
  cout << "SST Branch count = " << Branch_count << endl;
  cout << "MAX CLIQUE SIZE = " << Qmax.size() << endl;
  cout << "CLIQUE ELEMENT :" << endl;

  forall(v, Qmax){
    cout << " " << v;
  }

  cout << endl;

  return 0;

}


void dfmax(node_set R, node_set Q){

  node p, v;
  node_set Rp(G);

  while(!R.empty()){

    Branch_count++;

    p = R.choose();

    Q.insert(p);

    Rp.clear();
    //Rp = intersect(R, G.adj_nodes(p));
    forall_adj_nodes(v, p){
      if(R.member(v)){
	Rp.insert(v);
      }
    }

    if(!Rp.empty()){
      dfmax(Rp, Q);
    }
    else{
      if(Q.size() > Qmax.size()){
	Q.clear();
	forall(v, Q){
	  Qmax.insert(v);
	}
      }
    }
    
    Q.del(p);
    R.del(p);
  }

  return;

}
